package main

import (
	"context"
	"flag"
	"os"

	"code.fbi.h-da.de/danet/gosdn/application-framework/event"
	"code.fbi.h-da.de/danet/gosdn/application-framework/registration"
	"github.com/sirupsen/logrus"
)

func main() {
	controllerAddress := flag.String("controllerAddress", "localhost:55055", "The address to a goSDN controller to register at.")
	flag.Parse()

	// Register the application at the controller
	queueCredentials, err := registration.Register(context.Background(), *controllerAddress, "ws-events", "SecurePresharedToken")
	if err != nil {
		logrus.Errorf("failed to register application on control plane. %v", err)
		os.Exit(1)
	}

	// New EventService with topic ManagedNetworkElement (could also be
	// extended with other topics to subscribe to)
	eventService, err := event.NewEventService(
		queueCredentials,
		[]event.Topic{event.ManagedNetworkElement},
	)
	if err != nil {
		logrus.Errorf("failed to create event service. %v", err)
		os.Exit(1)
	}

	app := &Application{
		eventService: eventService,
		stopChannel:  make(chan os.Signal, 1),
	}

	// Run the Application
	if err := app.Run(); err != nil {
		logrus.Error(err)
	}
}
